local _, core = ...;
core.active = false;
local PlayerData = {};
local running = false;
local groupType = "RAID";
local MainFrame = nil;

local function NCreatePattern(pattern,maximize)
	pattern = string.gsub(pattern, "[%(%)%-%+%[%]]", "%%%1");
	if not maximize then
		pattern = string.gsub(pattern, "%%s", "(.-)");
	else
		pattern = string.gsub(pattern, "%%s", "(.+)");
	end
	pattern = string.gsub(pattern, "%%d", "%(%%d-%)");
	if not maximize then
		pattern = string.gsub(pattern, "%%%d%$s", "(.-)");
	else
		pattern = string.gsub(pattern, "%%%d%$s", "(.+)");
	end
	pattern = string.gsub(pattern, "%%%d$d", "%(%%d-%)");
	return pattern;
end

function core:Reset()
	PlayerData = {};
end

local function checkForRoll(name)
	for i = 1, #PlayerData, 1 do
		if (PlayerData[i].name == name) then
			return false;
		end
	end
	return true;
end

local function CountCoinsAndBijous()
	local result = {bijous = 0, coins = 0};
	for bag = 0,NUM_BAG_SLOTS do
		for slot = 1,GetContainerNumSlots(bag) do
			local itemID = GetContainerItemID(bag, slot);
			if (itemID and itemID >= 19698 and itemID <= 19706) then
				result.coins = result.coins + select(2, GetContainerItemInfo(bag, slot));
			end
			if (itemID and itemID >= 19707 and itemID <= 19715) then
				result.bijous = result.bijous + select(2, GetContainerItemInfo(bag, slot));
			end
		end
	end
	return result;
end

local function CalculateDistribution(items, groupSize)
	local bijousForEveryone = math.floor(items.bijous/groupSize);
	local bijousRest = items.bijous % groupSize;
	local coinsForEveryone = math.floor(items.coins/groupSize);
	local coinsRest = items.coins % groupSize;
	local result = {};
	if (bijousRest + coinsRest > groupSize) then
		result = {{players = bijousRest + coinsRest - groupSize, bijous = bijousForEveryone + 1, coins = coinsForEveryone + 1},
					{players = groupSize - coinsRest, bijous = bijousForEveryone + 1, coins = coinsForEveryone},
					{players = groupSize - bijousRest, bijous = bijousForEveryone, coins = coinsForEveryone + 1}};
	else
		if (bijousRest > 0) then
			result[1] = {players = bijousRest, bijous = bijousForEveryone + 1, coins = coinsForEveryone};
		end
		if (coinsRest > 0) then
			result[#result + 1] = {players = coinsRest, bijous = bijousForEveryone, coins = coinsForEveryone + 1};
		end
		if (bijousRest + coinsRest < groupSize) then
			result[#result + 1] = {players = groupSize - bijousRest - coinsRest, bijous = bijousForEveryone, coins = coinsForEveryone};
		end
	end
	return result;
end

local function insert(Table, n, entry)
	for i = #Table, n, -1 do
		Table[i+1] = Table[i];
	end
	Table[n] = entry;
end

local function delete(Table, n)
	for i = #Table - 1, n, -1 do
		Table[i] = Table[i + 1];
	end
end

local function SortPlayerData(coinsAndBijous)
	local PlayerDataSorted = {};
	local dist = CalculateDistribution(coinsAndBijous, #PlayerData);
	for x = 1, #PlayerData, 1 do
		local entry = PlayerData[x];
		local n = 1;
		local check = false;
		if (PlayerDataSorted[n]) then
			check = PlayerDataSorted[n].roll >= entry.roll;
		end
		while(check) do
			n = n + 1;
			if (PlayerDataSorted[n]) then
				check = PlayerDataSorted[n].roll >= entry.roll;
			else
				check = false;
			end
		end
		insert(PlayerDataSorted, n, entry);
	end
	local i = 1;
	for x = 1, #PlayerDataSorted, 1 do
		PlayerDataSorted[x].bijous = dist[i].bijous;
		PlayerDataSorted[x].coins = dist[i].coins;
		if (dist[i].players == 1) then
			i = i + 1;
		else
			dist[i].players = dist[i].players - 1;
		end
	end
	return PlayerDataSorted;
end

local function printInformations(sortedData)
	if (#sortedData > 0) then
		local minimum = sortedData[#sortedData].roll;
		local bijous = sortedData[#sortedData].bijous;
		local coins = sortedData[#sortedData].coins;
		for i = #sortedData - 1, 1, -1 do
			if (sortedData[i].bijous ~= bijous or sortedData[i].coins ~= coins) then
				SendChatMessage("Die Würfe " .. minimum .. " bis " .. sortedData[i+1].roll .. " erhalten " .. bijous .. " Schmuckstücke und " .. coins .. " Münzen.", groupType);
				minimum = sortedData[i].roll;
				bijous = sortedData[i].bijous;
				coins = sortedData[i].coins;
			end
		end
		SendChatMessage("Die Würfe " .. minimum .. " bis " .. sortedData[1].roll .. " erhalten " .. bijous .. " Schmuckstücke und " .. coins .. " Münzen.", groupType);
	end
end

local function BijouCoinCounter(arg, ...)
	if (GetNumGroupMembers() == 0) then
		print("Sie sind in keiner Gruppe");
	else
		if (IsInRaid()) then
			groupType = "RAID";
		else
			groupType = "PARTY";
		end
		if (arg == "start") then
			if (not core.active and not core.shown) then
					PlayerData = {};
					print("BijouCoinCounter gestartet. Nutzen Sie '/bcc stop' um die Würfelphase zu beenden und die Ergebnisse zu berrechnen");
					SendChatMessage("Bitte würfeln Sie jetzt!", groupType);
					core.active = true;
			else
				print("Die Würfelphase wurde bereits gestartet");
			end
		elseif (arg == "stop") then
			if (core.active) then
				core.active = false;
				local sorted = SortPlayerData(CountCoinsAndBijous());
				printInformations(sorted);
				MainFrame.show(sorted);
				SendChatMessage("Vielen Dank für die Teilnahme!", groupType);
			else
				print("Die Würfelphase wurde noch nicht gestartet");
			end
		else
			print("Bitte nutzen Sie '/bcc start' um die Würfelphase zu starten und '/bcc stop' um die Auswertung anzuzeigen.");
		end
	end
end

MainFrame = core:initFrame();

SLASH_CountCoinsBijous1 = "/bcc";
SlashCmdList.CountCoinsBijous = BijouCoinCounter;

local PatternRoll = NCreatePattern(RANDOM_ROLL_RESULT);
local LootS_EventFrame = CreateFrame("Frame");
LootS_EventFrame:RegisterEvent("CHAT_MSG_SYSTEM");
LootS_EventFrame:SetScript("OnEvent",
	function(_, _, message)
		if (core.active) then
			for name, roll, low, high in string.gmatch(message, PatternRoll) do
				local new = checkForRoll(name);
				if (new and low == "1" and high == "100") then
					PlayerData[#PlayerData+1] = {name = name, roll = tonumber(roll)};
				else
					SendChatMessage(name .. " hat versucht zu schummeln :P", groupType);
				end
			end
		end
	end
);